@@ -1,87 +1,65 @@
 ##############################################################################
-#      $URL: http://perlcritic.tigris.org/svn/perlcritic/tags/Perl-Critic-1.051/lib/Perl/Critic/Policy/Subroutines/ProhibitExcessComplexity.pm $
-#     $Date: 2007-04-12 01:26:09 -0700 (Thu, 12 Apr 2007) $
-#   $Author: thaljef $
-# $Revision: 1467 $
+#      $URL: http://perlcritic.tigris.org/svn/perlcritic/trunk/distributions/Perl-Critic/lib/Perl/Critic/Policy/Subroutines/ProhibitExcessComplexity.pm $
+#     $Date: 2010-06-22 16:14:07 -0400 (Tue, 22 Jun 2010) $
+#   $Author: clonezone $
+# $Revision: 3843 $
 ##############################################################################
 
 package Perl::Critic::Policy::Subroutines::ProhibitExcessComplexity;
 
+use 5.006001;
 use strict;
 use warnings;
+use Readonly;
+
 use Perl::Critic::Utils qw{ :severities :data_conversion :classification };
+use Perl::Critic::Utils::McCabe qw{ calculate_mccabe_of_sub };
+
 use base 'Perl::Critic::Policy';
 
-our $VERSION = 1.051;
+our $VERSION = '1.108';
 
 #-----------------------------------------------------------------------------
 
-my $expl = q{Consider refactoring};
-
-my @logic_ops = qw( && || ||= &&= or and xor ? <<= >>= );
-my %logic_ops = hashify( @logic_ops );
-
-my @logic_keywords = qw( if else elsif unless until while for foreach );
-my %logic_keywords = hashify( @logic_keywords );
+Readonly::Scalar my $EXPL => q{Consider refactoring};
 
 #-----------------------------------------------------------------------------
 
-sub supported_parameters { return qw( max_mccabe )                }
-sub default_severity  { return $SEVERITY_MEDIUM                }
-sub default_themes    { return qw(core complexity maintenance) }
-sub applies_to        { return 'PPI::Statement::Sub'           }
-
-#-----------------------------------------------------------------------------
-
-sub new {
-    my ( $class, %args ) = @_;
-    my $self = bless {}, $class;
-    $self->{_max_mccabe} = $args{max_mccabe} || 20;
-    return $self;
+sub supported_parameters {
+    return (
+        {
+            name            => 'max_mccabe',
+            description     => 'The maximum complexity score allowed.',
+            default_string  => '20',
+            behavior        => 'integer',
+            integer_minimum => 1,
+        },
+    );
 }
 
+sub default_severity { return $SEVERITY_MEDIUM                }
+sub default_themes   { return qw(core complexity maintenance) }
+sub applies_to       { return 'PPI::Statement::Sub'           }
+
 #-----------------------------------------------------------------------------
 
 sub violates {
     my ( $self, $elem, undef ) = @_;
 
-    my $count = 1; # Minimum score is 1
-    $count += _count_logic_keywords( $elem );
-    $count += _count_logic_operators( $elem );
+    my $score = calculate_mccabe_of_sub( $elem );
 
     # Is it too complex?
-    return if $count <= $self->{_max_mccabe};
+    return if $score <= $self->{_max_mccabe};
 
-    my $desc = qq{Subroutine with high complexity score ($count)};
-    return $self->violation( $desc, $expl, $elem );
-}
-
-#-----------------------------------------------------------------------------
-
-sub _count_logic_keywords {
-    my $elem = shift;  # Should be a PPI::Statement::Sub
-    my $count = 0;
-
-    my $keywords_ref = $elem->find('PPI::Token::Word');
-    if ( $keywords_ref ) { # should always be true due to "sub" keyword
-        my @filtered = grep { ! is_hash_key($_) } @{ $keywords_ref };
-        $count = grep { exists $logic_keywords{$_} } @filtered;
+    my $desc;
+    if ( my $name = $elem->name() ) {
+        $desc = qq<Subroutine "$name" with high complexity score ($score)>;
     }
-    return $count;
-}
-
-#-----------------------------------------------------------------------------
-
-sub _count_logic_operators {
-    my $elem = shift;  # Should be a PPI::Statement::Sub
-    my $count = 0;
-
-    my $operators_ref = $elem->find('PPI::Token::Operator');
-    if ( $operators_ref ) {
-        $count = grep { exists $logic_ops{$_} }  @{ $operators_ref };
+    else {
+        $desc = qq<Anonymous subroutine with high complexity score ($score)>;
     }
 
-    return $count;
+    return $self->violation( $desc, $EXPL, $elem );
 }
 
 
@@ -97,7 +75,13 @@ __END__
 
 =head1 NAME
 
-Perl::Critic::Policy::Subroutines::ProhibitExcessComplexity
+Perl::Critic::Policy::Subroutines::ProhibitExcessComplexity - Minimize complexity by factoring code into smaller subroutines.
+
+=head1 AFFILIATION
+
+This Policy is part of the core L<Perl::Critic|Perl::Critic>
+distribution.
+
 
 =head1 DESCRIPTION
 
@@ -109,7 +93,7 @@ possible paths through a subroutine.  This Policy approximates the
 McCabe score by summing the number of conditional statements and
 operators within a subroutine.  Research has shown that a McCabe score
 higher than 20 is a sign of high-risk, potentially untestable code.
-See L<http://www.sei.cmu.edu/str/descriptions/cyclomatic_body.html>
+See L<http://en.wikipedia.org/wiki/Cyclomatic_complexity>
 for some discussion about the McCabe number and other complexity
 metrics.
 
@@ -145,11 +129,11 @@ Just consider if there B<might> be a simpler way to get the job done.
 
 =head1 AUTHOR
 
-Jeffrey Ryan Thalhammer <thaljef@cpan.org>
+Jeffrey Ryan Thalhammer <jeff@imaginative-software.com>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005-2007 Jeffrey Ryan Thalhammer.  All rights reserved.
+Copyright (c) 2005-2010 Imaginative Software Systems.  All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.  The full text of this license
@@ -164,4 +148,4 @@ can be found in the LICENSE file included with this module.
 #   indent-tabs-mode: nil
 #   c-indentation-style: bsd
 # End:
-# ex: set ts=8 sts=4 sw=4 tw=78 ft=perl expandtab :
+# ex: set ts=8 sts=4 sw=4 tw=78 ft=perl expandtab shiftround :
