@@ -1,8 +1,8 @@
 ##############################################################################
-#      $URL: http://perlcritic.tigris.org/svn/perlcritic/tags/Perl-Critic-1.092/lib/Perl/Critic/TestUtils.pm $
-#     $Date: 2008-09-02 09:43:48 -0700 (Tue, 02 Sep 2008) $
-#   $Author: thaljef $
-# $Revision: 2721 $
+#      $URL: http://perlcritic.tigris.org/svn/perlcritic/trunk/distributions/Perl-Critic/lib/Perl/Critic/TestUtils.pm $
+#     $Date: 2010-06-22 16:14:07 -0400 (Tue, 22 Jun 2010) $
+#   $Author: clonezone $
+# $Revision: 3843 $
 ##############################################################################
 
 package Perl::Critic::TestUtils;
@@ -29,7 +29,7 @@ use Perl::Critic::Exception::Fatal::Internal qw{ &throw_internal };
 use Perl::Critic::Utils qw{ :severities :data_conversion policy_long_name };
 use Perl::Critic::PolicyFactory (-test => 1);
 
-our $VERSION = '1.092';
+our $VERSION = '1.108';
 
 Readonly::Array our @EXPORT_OK => qw(
     pcritique pcritique_with_violations
@@ -110,7 +110,7 @@ sub fcritique_with_violations {
     my $file = File::Spec->catfile($dir, @fileparts);
     if (open my $fh, '>', $file) {
         print {$fh} ${$code_ref};
-        close $fh or throw_generic "unable to close $file: $!";
+        close $fh or throw_generic "unable to close $file: $OS_ERROR";
     }
 
     # Use eval so we can clean up before throwing an exception in case of
@@ -132,24 +132,53 @@ sub fcritique {  ##no critic(ArgUnpacking)
     return scalar fcritique_with_violations(@_);
 }
 
+# Note: $include_extras is not documented in the POD because I'm not
+# committing to the interface yet.
 sub subtests_in_tree {
-    my $start = shift;
+    my ($start, $include_extras) = @_;
 
     my %subtests;
 
-    find( {wanted => sub {
-               return if ! -f $_;
-               my ($fileroot) = m{(.+)[.]run\z}xms;
-               return if !$fileroot;
-               my @pathparts = File::Spec->splitdir($fileroot);
-               if (@pathparts < 2) {
-                   throw_internal 'confusing policy test filename ' . $_;
-               }
-               my $policy = join q{::}, @pathparts[-2, -1]; ## no critic (MagicNumbers)
+    find(
+        {
+            wanted => sub {
+                return if not -f $_;
+
+                my ($fileroot) = m{(.+)[.]run\z}xms;
+
+                return if not $fileroot;
+
+                my @pathparts = File::Spec->splitdir($fileroot);
+                if (@pathparts < 2) {
+                    throw_internal 'confusing policy test filename ' . $_;
+                }
+
+                my $policy = join q{::}, @pathparts[-2, -1]; ## no critic (MagicNumbers)
+
+                my $globals = _globals_from_file( $_ );
+                if ( my $prerequisites = $globals->{prerequisites} ) {
+                    foreach my $prerequisite ( keys %{$prerequisites} ) {
+                        eval "require $prerequisite; 1" or return;
+                    }
+                }
+
+                my @subtests = _subtests_from_file( $_ );
+
+                if ($include_extras) {
+                    $subtests{$policy} =
+                        { subtests => [ @subtests ], globals => $globals };
+                }
+                else {
+                    $subtests{$policy} = [ @subtests ];
+                }
+
+                return;
+            },
+            no_chdir => 1,
+        },
+        $start
+    );
 
-               my @subtests = _subtests_from_file( $_ );
-               $subtests{ $policy } = [ @subtests ];
-           }, no_chdir => 1}, $start );
     return \%subtests;
 }
 
@@ -173,10 +202,44 @@ sub starting_points_including_examples {
     return (-e 'blib' ? 'blib' : 'lib', 'examples');
 }
 
+sub _globals_from_file {
+    my $test_file = shift;
+
+    my %valid_keys = hashify qw< prerequisites >;
+
+    return if -z $test_file;  # Skip if the Policy has a regular .t file.
+
+    my %globals;
+
+    open my $handle, '<', $test_file   ## no critic (RequireBriefOpen)
+        or throw_internal "Couldn't open $test_file: $OS_ERROR";
+
+    while ( my $line = <$handle> ) {
+        chomp;
+
+        if (
+            my ($key,$value) =
+                $line =~ m<\A [#][#] [ ] global [ ] (\S+) (?:\s+(.+))? >xms
+        ) {
+            next if not $key;
+            if ( not $valid_keys{$key} ) {
+                throw_internal "Unknown global key $key in $test_file";
+            }
+
+            if ( $key eq 'prerequisites' ) {
+                $value = { hashify( words_from_string($value) ) };
+            }
+            $globals{$key} = $value;
+        }
+    }
+    close $handle or throw_generic "unable to close $test_file: $OS_ERROR";
+
+    return \%globals;
+}
+
 # The internal representation of a subtest is just a hash with some
 # named keys.  It could be an object with accessors for safety's sake,
 # but at this point I don't see why.
-
 sub _subtests_from_file {
     my $test_file = shift;
 
@@ -185,17 +248,18 @@ sub _subtests_from_file {
     return if -z $test_file;  # Skip if the Policy has a regular .t file.
 
     open my $fh, '<', $test_file   ## no critic (RequireBriefOpen)
-      or throw_internal "Couldn't open $test_file: $OS_ERROR";
+        or throw_internal "Couldn't open $test_file: $OS_ERROR";
 
     my @subtests;
 
     my $incode = 0;
+    my $cut_in_code = 0;
     my $subtest;
     my $lineno;
     while ( <$fh> ) {
         ++$lineno;
         chomp;
-        my $inheader = /^## name/ .. /^## cut/; ## no critic(RegularExpression)
+        my $inheader = /^## name/ .. /^## cut/; ## no critic (ExtendedFormatting LineBoundaryMatching DotMatchAnything)
 
         my $line = $_;
 
@@ -215,6 +279,7 @@ sub _subtests_from_file {
                 }
                 $subtest->{lineno} = $lineno;
                 $incode = 0;
+                $cut_in_code = 0;
             }
             if ($incode) {
                 throw_internal "Header line found while still in code: $test_file";
@@ -223,15 +288,17 @@ sub _subtests_from_file {
         }
         elsif ( $subtest ) {
             $incode = 1;
-            # Don't start a subtest if we're not in one
-            push @{$subtest->{code}}, $line;
+            $cut_in_code ||= $line =~ m/ \A [#][#] [ ] cut \z /smx;
+            # Don't start a subtest if we're not in one.
+            # Don't add to the test if we have seen a '## cut'.
+            $cut_in_code or push @{$subtest->{code}}, $line;
         }
         elsif (@subtests) {
             ## don't complain if we have not yet hit the first test
             throw_internal "Got some code but I'm not in a subtest: $test_file";
         }
     }
-    close $fh or throw_generic "unable to close $test_file: $!";
+    close $fh or throw_generic "unable to close $test_file: $OS_ERROR";
     if ( $subtest ) {
         if ( $incode ) {
             push @subtests, _finalize_subtest( $subtest );
@@ -273,7 +340,7 @@ sub _finalize_subtest {
 
     if (defined $subtest->{error}) {
         if ( $subtest->{error} =~ m{ \A / (.*) / \z }xms) {
-            $subtest->{error} = eval {qr/$1/}; ##no critic (RegularExpressions::)
+            $subtest->{error} = eval {qr/$1/}; ## no critic (ExtendedFormatting LineBoundaryMatching DotMatchAnything)
             if ($EVAL_ERROR) {
                 throw_internal
                     "$subtest->{name} 'error' has a malformed regular expression";
@@ -289,7 +356,8 @@ sub bundled_policy_names {
     my $manifest = ExtUtils::Manifest::maniread();
     my @policy_paths = map {m{\A lib/(Perl/Critic/Policy/.*).pm \z}xms} keys %{$manifest};
     my @policies = map { join q{::}, split m{/}xms, $_} @policy_paths;
-    return sort @policies;
+    my @sorted_policies = sort @policies;
+    return @sorted_policies;
 }
 
 sub names_of_policies_willing_to_work {
@@ -311,13 +379,19 @@ __END__
 
 =pod
 
-=for stopwords subtest subtests
+=for stopwords RCS subtest subtests
 
 =head1 NAME
 
 Perl::Critic::TestUtils - Utility functions for testing new Policies.
 
 
+=head1 INTERFACE SUPPORT
+
+This is considered to be a public module.  Any changes to its
+interface will go through a deprecation cycle.
+
+
 =head1 SYNOPSIS
 
     use Perl::Critic::TestUtils qw(critique pcritique fcritique);
@@ -346,7 +420,7 @@ Perl::Critic::TestUtils - Utility functions for testing new Policies.
 
 This module is used by L<Perl::Critic|Perl::Critic> only for
 self-testing. It provides a few handy subroutines for testing new
-Perl::Critic::Policy modules.  Look at the test scripts that ship with
+Perl::Critic::Policy modules.  Look at the test programs that ship with
 Perl::Critic for more examples of how to use these subroutines.
 
 
@@ -530,6 +604,14 @@ C<fcritique> for more details):
 The value of C<parms> will get C<eval>ed and passed to C<pcritique()>,
 so be careful.
 
+In general, a subtest document runs from the C<## cut> that starts it to
+either the next C<## name> or the end of the file. In very rare circumstances
+you may need to end the test document earlier. A second C<## cut> will do
+this. The only known need for this is in
+F<t/Miscellanea/RequireRcsKeywords.run>, where it is used to prevent the RCS
+keywords in the file footer from producing false positives or negatives in the
+last test.
+
 Note that nowhere within the F<.run> file itself do you specify the
 policy that you're testing.  That's implicit within the filename.
 
@@ -551,7 +633,7 @@ and the rest of the L<Perl::Critic|Perl::Critic> team.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005-2008 Chris Dolan.  All rights reserved.
+Copyright (c) 2005-2010 Chris Dolan.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.  The full text of this license
